import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import { useEffect } from 'react';
import { gql } from 'graphql-request';
import { useQuery } from 'react-query';
import { useGraphqlClient } from './use-graphql-client';
const fieldFragment = `
  name
  type {
    gqlType
    isArray
    modifier
    pgAlias
    pgType
    subtype
    typmod
  }
`;
const queryFragment = `
  query {
    all
    create
    delete
    one
    update
  }
`;
const primaryConstraintsFragment = `
  primaryKeyConstraints {
    name
    fields {
      ${fieldFragment}
    }
  }
`;
const foreignKeyConstraintsFragments = `
  foreignKeyConstraints {
    name
    fields {
      ${fieldFragment}
    }
    refFields {
      ${fieldFragment}
    }
    refTable {
      name
    }
  }
`;
const inflectionFragment = `
  inflection {
    allRows
    allRowsSimple
    conditionType
    connection
    createField
    createInputType
    createPayloadType
    deleteByPrimaryKey
    deletePayloadType
    edge
    edgeField
    enumType
    filterType
    inputType
    orderByType
    patchField
    patchType
    tableFieldName
    tableType
    typeName
    updateByPrimaryKey
    updatePayloadType
  }
`;
const metaQuery = gql`
  query SchemaMetaQuery {
    _meta {
      tables {
        name
        ${queryFragment}

        ${inflectionFragment}

        fields {
          ${fieldFragment}
        }

        ${primaryConstraintsFragment}

        ${foreignKeyConstraintsFragments}

        uniqueConstraints {
          name
          fields {
            ${fieldFragment}
          }
        }

        relations {
          belongsTo {
            fieldName
            isUnique
            type
            keys {
              ${fieldFragment}
            }
            references {
              name
              fields {
                ${fieldFragment}
              }
            }
          }
          has {
            fieldName
            isUnique
            type
            keys {
              ${fieldFragment}
            }
            referencedBy {
              name
              fields {
                ${fieldFragment}
              }
              ${primaryConstraintsFragment}
              ${foreignKeyConstraintsFragments}
            }
          }
          hasMany {
            fieldName
            isUnique
            type
            keys {
              ${fieldFragment}
            }
            referencedBy {
              name
              fields {
                ${fieldFragment}
              }
              ${primaryConstraintsFragment}
              ${foreignKeyConstraintsFragments}
            }
          }
          hasOne {
            fieldName
            isUnique
            type
            keys {
              ${fieldFragment}
            }
            referencedBy {
              name
              fields {
                ${fieldFragment}
              }
              ${primaryConstraintsFragment}
              ${foreignKeyConstraintsFragments}
            }
          }
          manyToMany {
            fieldName
            type
            leftKeyAttributes {
              ${fieldFragment}
            }
            rightKeyAttributes {
              ${fieldFragment}
            }
            junctionTable {
              name
              fields {
                ${fieldFragment}
              }
              ${queryFragment}
              ${primaryConstraintsFragment}
              ${foreignKeyConstraintsFragments}
            }
            rightTable {
              name
              fields {
                ${fieldFragment}
              }
              ${queryFragment}
              ${primaryConstraintsFragment}
              ${foreignKeyConstraintsFragments}
            }
          }
        }
      }
    }
  }
`;
export function useSchemaMeta(_ref = {}) {
  let {
    tableName,
    onSuccess,
    onError,
    headers
  } = _ref,
      restOptions = _objectWithoutProperties(_ref, ["tableName", "onSuccess", "onError", "headers"]);

  const graphqlClient = useGraphqlClient();
  useEffect(() => {
    if (headers && graphqlClient) {
      graphqlClient.setHeaders(headers);
    }
  }, [graphqlClient, headers]);

  const _useQuery = useQuery('schemaMeta', async () => {
    return await graphqlClient.request(metaQuery);
  }, _objectSpread(_objectSpread({
    onError,
    onSuccess
  }, restOptions), {}, {
    // The query will not execute until the graphqlClient exists
    enabled: !!graphqlClient,
    // SQL schema rarely changes, so we don't want it to invalidate too often
    refetchInterval: false,
    refetchIntervalInBackground: false,
    refetchOnMount: false,
    refetchOnWindowFocus: false
  })),
        {
    data,
    refetch
  } = _useQuery,
        otherProps = _objectWithoutProperties(_useQuery, ["data", "refetch"]);

  return _objectSpread(_objectSpread({}, otherProps), {}, {
    refetch,
    data: tableName ? data?._meta?.tables[tableName] : data?._meta
  });
}